/*
 *  Copyright (C) 2007 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_ttf.h>
#include "gp2x_psp.h"

#include "psp_global.h"
#include "psp_danzeff.h"
#include "psp_sdl.h"
#include "psp_menu.h"
#include "psp_help.h"
#include "psp_fmgr.h"
#include "psp_editor.h"

# define MENU_FILENAME     0
# define MENU_WIDTH        1
# define MENU_HEIGHT       2
# define MENU_MOVE_CURSOR  3
# define MENU_FG_COLOR     4
# define MENU_BG_COLOR     5
# define MENU_DOS_MODE     6
# define MENU_EXPAND_TAB   7
# define MENU_TAB_SIZE     8

# define MENU_HELP         9
# define MENU_NEW         10
# define MENU_LOAD        11
# define MENU_SAVE        12

# define MENU_BACK        13
# define MENU_EXIT        14

# define MAX_MENU_ITEM (MENU_EXIT + 1)

  static menu_item_t menu_list[] =
  {
    { "Filename   : " },
    { "Width      : " },
    { "Height     : " },
    { "Cursor     : " },
    { "Text color : " },
    { "Back color : " },
    { "Dos mode   : " },
    { "Expand tab : " },
    { "Tab size   : " },

    { "Help" },

    { "New" },
    { "Load" },
    { "Save" },
    { "Back" },

    { "Exit" }
  };

  static int cur_menu_id = MENU_NEW;
  static int dos_mode = 0;
  static int expand_tab = 0;
  static int tab_stop = 0;
  static int move_on_text = 0;
  static int fg_color = 0;
  static int bg_color = 0;
  static int screen_w = 0;
  static int screen_h = 0;

void
string_fill_with_space(char *buffer, int size)
{
  int length = strlen(buffer);
  int index;

  for (index = length; index < size; index++) {
    buffer[index] = ' ';
  }
  buffer[size] = 0;
}

# define FILENAME_FIELD_WIDTH      35

static void 
psp_display_screen_menu(void)
{
  char buffer[512];

  int menu_id = 0;
  int color   = 0;
  int x       = 0;
  int y       = 0;
  int y_step  = 0;

  {
    psp_sdl_blit_menu();

    psp_sdl_back_print(240, 225, "By Zx-81", GP2X_MENU_AUTHOR_COLOR);
  }
  
  x      = 10;
  y      = 10;
  y_step = 10;
  
  for (menu_id = 0; menu_id < MAX_MENU_ITEM; menu_id++) {
    color = GP2X_MENU_TEXT_COLOR;
    if (cur_menu_id == menu_id) color = GP2X_MENU_SEL_COLOR;
    else 
    if (menu_id == MENU_EXIT) color = GP2X_MENU_WARNING_COLOR;
    else
    if (menu_id == MENU_HELP) color = GP2X_MENU_GREEN_COLOR;

    psp_sdl_back_print(x, y, menu_list[menu_id].title, color);

    if (menu_id == MENU_FILENAME) {
      int length = strlen(PSPWRITE.edit_filename);
      int first = 0;
      if (length > (FILENAME_FIELD_WIDTH-1)) {
        first = length - (FILENAME_FIELD_WIDTH-1);
      }
      sprintf(buffer, "%s", PSPWRITE.edit_filename + first);
      if (menu_id == cur_menu_id) strcat(buffer, "_");
      string_fill_with_space(buffer, FILENAME_FIELD_WIDTH+1);
      psp_sdl_back_print(90, y, buffer, color);

    } else
    if (menu_id == MENU_MOVE_CURSOR) {
      if (move_on_text) strcpy(buffer, "only on text");
      else              strcpy(buffer, "everywhere");
      string_fill_with_space(buffer, 32);
      psp_sdl_back_print(90, y, buffer, color);

    } else
    if (menu_id == MENU_WIDTH) {
      sprintf(buffer, "%d", screen_w);
      string_fill_with_space(buffer, 32);
      psp_sdl_back_print(90, y, buffer, color);

    } else
    if (menu_id == MENU_HEIGHT) {
      sprintf(buffer, "%d", screen_h);
      string_fill_with_space(buffer, 32);
      psp_sdl_back_print(90, y, buffer, color);

    } else
    if (menu_id == MENU_FG_COLOR) {
      sprintf(buffer,"%s", editor_colors_name[fg_color]);
      color = editor_colors[fg_color];
      string_fill_with_space(buffer, 10);
      psp_sdl_back_print(90, y, buffer, color);

    } else
    if (menu_id == MENU_BG_COLOR) {
      sprintf(buffer,"%s", editor_colors_name[bg_color]);
      color = editor_colors[bg_color];
      string_fill_with_space(buffer, 32);
      psp_sdl_back_print(90, y, buffer, color);

    } else
    if (menu_id == MENU_DOS_MODE) {
      if (dos_mode) strcpy(buffer, "yes");
      else                  strcpy(buffer, "no");
      string_fill_with_space(buffer, 10);
      psp_sdl_back_print(90, y, buffer, color);
    } else
    if (menu_id == MENU_EXPAND_TAB) {
      if (expand_tab) strcpy(buffer, "yes");
      else                     strcpy(buffer, "no");
      string_fill_with_space(buffer, 32);
      psp_sdl_back_print(90, y, buffer, color);
    } else
    if (menu_id == MENU_TAB_SIZE) {
      sprintf(buffer, "%d", tab_stop);
      string_fill_with_space(buffer, 10);
      psp_sdl_back_print(90, y, buffer, color);
      y += y_step;

    } else
    if (menu_id == MENU_HELP) {
      y += y_step;
    } else
    if (menu_id == MENU_BACK) {
      y += y_step;
    }

    y += y_step;
  }
}

static void
psp_edit_menu_validate(void)
{
  PSPWRITE.dos_mode      = dos_mode;
  PSPWRITE.expand_tab    = expand_tab;
  PSPWRITE.tab_stop      = tab_stop;
  PSPWRITE.move_on_text  = move_on_text;
  PSPWRITE.fg_color      = fg_color;
  PSPWRITE.bg_color      = bg_color;
  PSPWRITE.screen_h      = screen_h;
  PSPWRITE.screen_w      = screen_w;
}

static void
psp_edit_menu_init()
{
  dos_mode      = PSPWRITE.dos_mode;
  expand_tab    = PSPWRITE.expand_tab;
  tab_stop      = PSPWRITE.tab_stop;
  move_on_text  = PSPWRITE.move_on_text;
  fg_color      = PSPWRITE.fg_color;
  bg_color      = PSPWRITE.bg_color;
  screen_h      = PSPWRITE.screen_h;
  screen_w      = PSPWRITE.screen_w;
}

static void
psp_edit_menu_save_config()
{
  int error;

  psp_edit_menu_validate();

  error = psp_global_save_config();

  if (! error) /* save OK */
  {
    psp_display_screen_menu();
    psp_sdl_back_print(150, 60, "Config file saved !", 
                       GP2X_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  else 
  {
    psp_display_screen_menu();
    psp_sdl_back_print(150, 60, "Can't save config file !", 
                       GP2X_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
}

int
psp_edit_menu_exit(void)
{
  if (! psp_edit_menu_confirm_save()) {
    return 0;
  }

  gp2xCtrlData c;

  psp_display_screen_menu();
  psp_sdl_back_print(150, 60, "Press X to exit !", GP2X_MENU_WARNING_COLOR);
  psp_sdl_back_print(150, 70, "B save config & exit", GP2X_MENU_WARNING_COLOR);
  psp_sdl_flip();

  psp_kbd_wait_no_button();

  do
  {
    gp2xCtrlReadBufferPositive(&c, 1);

    if (c.Buttons & GP2X_CTRL_CROSS) {
      psp_sdl_exit(0);
    } else
    if (c.Buttons & GP2X_CTRL_CIRCLE) {
      psp_edit_menu_save_config();
      psp_sdl_exit(0);
    }

  } while (c.Buttons == 0);

  psp_kbd_wait_no_button();

  return 0;
}

int
psp_edit_menu_confirm_overwrite()
{
  gp2xCtrlData c;

  if (PSPWRITE.ask_overwrite) {

    struct stat aStat;
    if (stat(PSPWRITE.edit_filename, &aStat)) {
      PSPWRITE.ask_overwrite = 0;
      return 1;
    }

    psp_display_screen_menu();
    psp_sdl_back_print(150, 60, "X overwrite ? B cancel", GP2X_MENU_WARNING_COLOR);
    psp_sdl_flip();

    psp_kbd_wait_no_button();
    do
    {
      gp2xCtrlReadBufferPositive(&c, 1);
      c.Buttons &= GP2X_ALL_BUTTON_MASK;

# ifdef USE_GP2X_IRKEYB
      psp_irkeyb_set_psp_key(&c);
# endif

      if (c.Buttons & GP2X_CTRL_CROSS) {
        return 1;
      } else 
      if (c.Buttons & GP2X_CTRL_SQUARE) {
        return 0;
      }

    } while (c.Buttons == 0);
    psp_kbd_wait_no_button();
  }
  return 0;
}

static int
psp_edit_menu_save()
{
  int error;
  if (PSPWRITE.ask_overwrite) {
    if (! psp_edit_menu_confirm_overwrite()) {
      return 0;
    }
  }
  PSPWRITE.ask_overwrite = 0;
  error = psp_editor_save();

  if (! error) /* save OK */
  {
    psp_display_screen_menu();
    psp_sdl_back_print(150, 60,  "File saved !", 
                       GP2X_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  else 
  {
    psp_display_screen_menu();
    psp_sdl_back_print(150, 60,  "Can't save file !", 
                       GP2X_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  return error;
}

int
psp_edit_menu_confirm_save()
{
  gp2xCtrlData c;

  if (PSPWRITE.is_modified) {

    psp_display_screen_menu();
    psp_sdl_back_print(150, 60, "Save changes ? X Save ", GP2X_MENU_WARNING_COLOR);
    psp_sdl_back_print(150, 70, "B ignore, A cancel", GP2X_MENU_WARNING_COLOR);
    psp_sdl_flip();

    psp_kbd_wait_no_button();
    do
    {
      gp2xCtrlReadBufferPositive(&c, 1);

      if (c.Buttons & GP2X_CTRL_CROSS) {
        if (psp_edit_menu_save()) {
          /* failed -> cancel */
          return 0;
        }
      } else 
      if (c.Buttons & GP2X_CTRL_SQUARE) {
        return 0;
      } else 
      if (c.Buttons & GP2X_CTRL_CIRCLE) {
        break;
      }

    } while (c.Buttons == 0);
    psp_kbd_wait_no_button();
  }
  return 1;
}

int
psp_edit_menu_new(void)
{
  if (psp_edit_menu_confirm_save()) {
    psp_editor_new();
  }
  return 0;
}

int
psp_edit_menu_load(void)
{
  int ret;

  if (! psp_edit_menu_confirm_save()) {
    return 0;
  }
  ret = psp_fmgr_menu(GP2X_FMGR_FORMAT_TXT);
  if (ret ==  1) /* load OK */
  {
    psp_display_screen_menu();
    psp_sdl_back_print(150,  60, "File loaded !", GP2X_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
    return 1;
  }
  else 
  if (ret == -1) /* Load Error */
  {
    psp_display_screen_menu();
    psp_sdl_back_print(150,  60, "Can't load file !", GP2X_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  return 0;
}

void
psp_edit_menu_color( int step )
{
  if (step < 0) {
    fg_color--;
    if (fg_color < 0) fg_color = COLOR_IMAGE-1;
  } else {
    fg_color++;
    if (fg_color >= COLOR_IMAGE) fg_color = 0;
  }
}

void
psp_edit_menu_width( int step )
{
  if (step < 0) {
    screen_w--;
    if (screen_w < EDITOR_SCREEN_MIN_WIDTH) {
      screen_w = EDITOR_SCREEN_WIDTH;
    }
  } else {
    screen_w++;
    if (screen_w > EDITOR_SCREEN_WIDTH) {
      screen_w = EDITOR_SCREEN_MIN_WIDTH;
    }
  }
}

void
psp_edit_menu_height( int step )
{
  if (step > 0) {
    screen_h++;
    if (screen_h > EDITOR_SCREEN_HEIGHT) {
      screen_h = EDITOR_SCREEN_MIN_HEIGHT;
    }
  } else {
    screen_h--;
    if (screen_h < EDITOR_SCREEN_MIN_HEIGHT) {
      screen_h = EDITOR_SCREEN_HEIGHT;
    }
  }
}

void
psp_edit_menu_bg_color( int step )
{
  if (step > 0) {
    bg_color++;
    if (bg_color >= EDITOR_MAX_COLOR) bg_color = 0;
  } else {
    bg_color--;
    if (bg_color < 0) bg_color = EDITOR_MAX_COLOR-1;
  }
}


void
psp_edit_menu_tabsize( int step )
{
  if (step < 0) {
    tab_stop--;
    if (tab_stop < 1) tab_stop = 10;
  } else {
    tab_stop++;
    if (tab_stop > 10) tab_stop = 1;
  }
}

void
psp_edit_menu_filename_add_char(int a_char)
{
  int length = strlen(PSPWRITE.edit_filename);
  if (length < (MAX_PATH-2)) {
    PSPWRITE.edit_filename[length] = a_char;
    PSPWRITE.edit_filename[length+1] = 0;
  }
}

void
psp_edit_menu_filename_clear()
{
  PSPWRITE.edit_filename[0] = 0;
}

void
psp_edit_menu_filename_del()
{
  int length = strlen(PSPWRITE.edit_filename);
  if (length >= 1) {
    PSPWRITE.edit_filename[length - 1] = 0;
  }
}

int 
psp_edit_main_menu(void)
{
  gp2xCtrlData c;
  long        new_pad;
  long        old_pad;

  int         last_time;
  int         end_menu;

  int         danzeff_mode = 0;
  int         danzeff_key;

  int         step = 0;

  psp_kbd_wait_no_button();
  psp_edit_menu_init();

  old_pad   = 0;
  last_time = 0;
  end_menu  = 0;

  while (! end_menu)
  {
    psp_display_screen_menu();

    if (danzeff_mode) {
      danzeff_moveTo(0, 0);
      danzeff_render();
    }
    psp_sdl_flip();

    while (1) {

      gp2xCtrlPeekBufferPositive(&c, 1);

      new_pad = c.Buttons;

      if ((old_pad != new_pad) || ((c.TimeStamp - last_time) > GP2X_MENU_MIN_TIME)) {
        last_time = c.TimeStamp;
        old_pad = new_pad;
        break;
      }
    }

    if (danzeff_mode) {

      danzeff_key = danzeff_readInput(c);
      if (danzeff_key > DANZEFF_START) {
        /* Disable utf8 */
        danzeff_key &= 0x7f;
        if (danzeff_key > ' ') {
          psp_edit_menu_filename_add_char(danzeff_key);
        } else
        if (danzeff_key == DANZEFF_CLEAR) {
          psp_edit_menu_filename_clear();
        } else
        if (danzeff_key == DANZEFF_DEL) {
          psp_edit_menu_filename_del();
        }

      } else 
      if ((danzeff_key == DANZEFF_START ) || 
          (danzeff_key == DANZEFF_SELECT)) 
      {
        danzeff_mode = 0;
        old_pad = new_pad = 0;

        psp_kbd_wait_no_button();
      }
      continue;
    }

    if ((c.Buttons & (GP2X_CTRL_LTRIGGER|GP2X_CTRL_RTRIGGER|GP2X_CTRL_START)) ==
        (GP2X_CTRL_LTRIGGER|GP2X_CTRL_RTRIGGER|GP2X_CTRL_START)) {
      /* Exit ! */
      psp_sdl_exit(0);
    } else
    if ((new_pad & GP2X_CTRL_CROSS ) || 
        (new_pad & GP2X_CTRL_CIRCLE))
    {
      if (new_pad & (GP2X_CTRL_RIGHT|GP2X_CTRL_CROSS)) {
        step = 1;
      } else
      if (new_pad & (GP2X_CTRL_LEFT|GP2X_CTRL_CIRCLE)) {
        step = -1;
      }

      switch (cur_menu_id ) 
      {
        case MENU_LOAD      : if (psp_edit_menu_load()) {
                                end_menu = 1;
                              }
        break;              

        case MENU_NEW       : psp_edit_menu_new();
                              end_menu = 1;
        break;              

        case MENU_SAVE      : psp_edit_menu_save();
        break;              

        case MENU_BACK      : end_menu = 1;
        break;

        case MENU_EXIT      : psp_edit_menu_exit();
        break;

        case MENU_HELP      : psp_help_menu();
                              old_pad = new_pad = 0;
        break;              

        case MENU_TAB_SIZE : psp_edit_menu_tabsize( step );
        break;              
        case MENU_FG_COLOR : psp_edit_menu_color( step );
        break;              
        case MENU_BG_COLOR : psp_edit_menu_bg_color( step );
        break;              
        case MENU_WIDTH    : psp_edit_menu_width( step );
        break;              
        case MENU_HEIGHT   : psp_edit_menu_height( step );
        break;              
        case MENU_DOS_MODE : dos_mode = ! dos_mode;
        break;              
        case MENU_MOVE_CURSOR : move_on_text = ! move_on_text;
        break;              
        case MENU_EXPAND_TAB : expand_tab = ! expand_tab;
        break;              
        case MENU_FILENAME : if (step < 0) psp_edit_menu_filename_del();
        break;
      }

    } else
    if(new_pad & GP2X_CTRL_START) {
      danzeff_mode = 1;
    } else
    if(new_pad & GP2X_CTRL_UP) {

      if (cur_menu_id > 0) cur_menu_id--;
      else                 cur_menu_id = MAX_MENU_ITEM-1;

    } else
    if(new_pad & GP2X_CTRL_DOWN) {

      if (cur_menu_id < (MAX_MENU_ITEM-1)) cur_menu_id++;
      else                                 cur_menu_id = 0;

    } else  
    if(new_pad & GP2X_CTRL_SQUARE) {
      /* Cancel */
      end_menu = -1;
    } else 
    if(new_pad & GP2X_CTRL_SELECT) {
      /* Back to Console */
      end_menu = 1;
    } else
    if(new_pad & GP2X_CTRL_RTRIGGER) {
      /* TO_BE_DONE ! */
    } else
    if(new_pad & GP2X_CTRL_LTRIGGER) {
      /* TO_BE_DONE ! */
    }
  }

  if (end_menu > 0) {
    psp_edit_menu_validate();
  }
 
  psp_editor_update_column();
  psp_editor_update_line();

  psp_kbd_wait_no_button();

  return (end_menu == 2);
}

